"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dolphin_js_1 = require("dolphin-js");
const FakeMemory_1 = require("modloader64_api/SidedProxy/FakeMemory");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const GCRomHeader_1 = require("./GCRomHeader");
const DolphinMemory_1 = require("./DolphinMemory");
const EventHandler_1 = require("modloader64_api/EventHandler");
const ml64tk_1 = require("ml64tk");
const getDolphinLibraryPath_1 = require("./getDolphinLibraryPath");
const ImGuiAppImpl_1 = require("./ImGuiAppImpl");
const worker_threads_1 = __importDefault(require("worker_threads"));
class DolphinConsole {
    startInfo = { isConfigure: false, gameFilePath: "", lobby: "" };
    romName;
    rom;
    mem;
    frame = 0;
    callbacks = new Map();
    lobby;
    logger;
    constructor(rom, logger, lobby, config) {
        this.rom = fs_1.default.readFileSync(rom);
        this.romName = path_1.default.parse(rom).base;
        this.startInfo.gameFilePath = rom;
        this.logger = logger;
        this.lobby = lobby;
        this.startInfo.lobby = lobby;
        this.mem = new DolphinMemory_1.DolphinMemory();
        EventHandler_1.bus.on("DOLPHIN_ENABLE_PATCH", (evt) => {
            dolphin_js_1.Util.enablePatch(new dolphin_js_1.UICommon.GameFile(rom), evt);
        });
        EventHandler_1.bus.on("DOLPHIN_ENABLE_GECKO", (evt) => {
            dolphin_js_1.Util.enableGeckoCode(new dolphin_js_1.UICommon.GameFile(rom), evt);
        });
        EventHandler_1.bus.on("DOLPHIN_ENABLE_ARCODE", (evt) => {
            dolphin_js_1.Util.enableARCode(new dolphin_js_1.UICommon.GameFile(rom), evt);
        });
        if (global.ModLoader.hasOwnProperty("EXTRA_DATA")) {
            this.startInfo.isConfigure = global.ModLoader["EXTRA_DATA"].toUpperCase() === "TRUE";
        }
    }
    onNewFrame() {
        if (this.callbacks.has("vi-update" /* Emulator_Callbacks.vi_update */)) {
            this.callbacks.get("vi-update" /* Emulator_Callbacks.vi_update */).forEach((fn) => {
                fn(this.frame);
            });
        }
    }
    startEmulator(preStartCallback) {
        let buf = preStartCallback();
        dolphin_js_1.Dolphin.loadLibrary({
            libraryPath: (0, getDolphinLibraryPath_1.getDolphinLibraryPath)()
        });
        const hostWorker = new worker_threads_1.default.Worker(path_1.default.join(__dirname, 'DolphinHostThread.js'), { workerData: { startinfo: this.startInfo, buf: buf, romName: this.romName } });
        if (!this.startInfo.isConfigure) {
            const app = new ImGuiAppImpl_1.ImGuiAppImpl();
            app.framecallback = this.onNewFrame.bind(this);
            app.run();
            app.setHostWorker(hostWorker);
            let processFrame;
            hostWorker.on('message', value => {
                if (value.msg == 'hostReady') {
                    processFrame = setInterval(() => {
                        dolphin_js_1.Dolphin.handleFrame(() => {
                            if (this.callbacks.has("new-frame" /* Emulator_Callbacks.new_frame */)) {
                                this.callbacks.get("new-frame" /* Emulator_Callbacks.new_frame */).forEach((fn) => {
                                    fn(this.frame);
                                });
                            }
                        });
                    }, 1);
                    dolphin_js_1.Dolphin.enableFrameHandler(true);
                }
                else if (value.msg == 'toggleImGuiVisibility') {
                    const checked = value.data;
                    if (checked)
                        app.show();
                    else
                        app.hide();
                }
            });
            hostWorker.on('exit', () => {
                if (!this.startInfo.isConfigure)
                    clearInterval(processFrame);
                app.close();
                EventHandler_1.bus.emit('SHUTDOWN_EVERYTHING', {});
                setTimeout(() => {
                    process.exit(0);
                }, 3000);
            });
        }
        //@ts-ignore
        this.rom = undefined;
        return this.mem;
    }
    stopEmulator() {
        dolphin_js_1.Dolphin.shutdown();
    }
    softReset() {
    }
    hardReset() {
    }
    saveState(file) {
        dolphin_js_1.State.saveAs(file);
    }
    loadState(file) {
        dolphin_js_1.State.loadAs(file);
    }
    finishInjects() {
    }
    isEmulatorReady() {
        return dolphin_js_1.Core.isRunning();
    }
    getLoadedRom() {
        return this.rom;
    }
    getRomOriginalSize() {
        return this.rom.byteLength;
    }
    getRomHeader() {
        let head = new GCRomHeader_1.GCRomHeader();
        head.id = this.rom.slice(0, 4).toString();
        let b = -1;
        let o = 0x20;
        while (b !== 0) {
            b = this.rom.readUInt8(o);
            head.name += this.rom.slice(o, o + 1).toString();
            o++;
        }
        head.name = head.name.trim();
        return head;
    }
    pauseEmulator() {
    }
    resumeEmulator() {
    }
    getMemoryAccess() {
        return this.mem;
    }
    setSaveDir(path) {
    }
    getUtils() {
        return {};
    }
    getFrameCount() {
        return this.frame;
    }
    setFrameCount(num) {
        this.frame = num;
    }
    registerCallback(type, callback) {
        if (!this.callbacks.has(type)) {
            this.callbacks.set(type, []);
        }
        this.callbacks.get(type).push(callback);
    }
    on(which, callback) {
        this.registerCallback(which, callback);
    }
    getSaveStateManager() {
        return {};
    }
    getImGuiAccess() {
        return ml64tk_1.ImGui;
    }
    getSDLAccess() {
        return {};
    }
    getGfxAccess() {
        return {};
    }
    getInputAccess() {
        return {};
    }
    getYaz0Encoder() {
        return {};
    }
    getDebuggerAccess() {
        return {};
    }
    getHiResTextureAccess() {
        return {};
    }
    getInternalPlugin() {
        return "";
    }
    getRomAccess() {
        return new FakeMemory_1.FakeRom();
    }
}
exports.default = DolphinConsole;
//# sourceMappingURL=DolphinConsole.js.map