"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const worker_threads_1 = __importDefault(require("worker_threads"));
const dolphin_js_1 = require("dolphin-js");
const getDolphinUserDirectoryPath_1 = require("./getDolphinUserDirectoryPath");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const startInfo = worker_threads_1.default.workerData.startinfo;
const romName = worker_threads_1.default.workerData.romName;
let buf = worker_threads_1.default.workerData.buf;
let ndir = fs_1.default.mkdtempSync('ModLoader64_temp_');
fs_1.default.writeFileSync(path_1.default.resolve(ndir, romName), buf);
buf.fill(0);
buf = undefined;
startInfo.gameFilePath = path_1.default.resolve(ndir, romName);
dolphin_js_1.Dolphin.startup({
    applicationDisplayName: 'ModLoader64',
    userDirectoryPath: (0, getDolphinUserDirectoryPath_1.getDolphinUserDirectoryPath)()
}, () => {
    dolphin_js_1.Config.setBool('-MAIN_USE_PANIC_HANDLERS', false);
    dolphin_js_1.Config.setBool('-Main,Interface.PlayMode', !startInfo.isConfigure);
    dolphin_js_1.Config.setBool('-Main,Display.RenderToMain', !startInfo.isConfigure);
    dolphin_js_1.Config.setBool('-Main,Interface.HideFPSInfo', false);
    dolphin_js_1.Config.setString('@GCIPathOverride', 0 /* Enums.ExpansionInterface.Slot.A */, path_1.default.resolve(`./saves/${startInfo.lobby}`));
});
const processUI = setInterval(() => {
    dolphin_js_1.Dolphin.processOne();
    if (dolphin_js_1.Gui.Application.hasExited()) {
        clearInterval(processUI);
        if (!startInfo.isConfigure)
            dolphin_js_1.Dolphin.enableFrameHandler(false);
        dolphin_js_1.Dolphin.shutdown();
        worker_threads_1.default.parentPort?.close();
    }
}, 16);
dolphin_js_1.Gui.MainWindow.setIcon('assets/icon.png');
dolphin_js_1.Gui.MainWindow.show();
dolphin_js_1.Gui.Settings.setToolBarVisible(startInfo.isConfigure);
dolphin_js_1.Gui.Settings.setDebugModeEnabled(false);
dolphin_js_1.Gui.Settings.setCheatsEnabled(true);
if (!startInfo.isConfigure) {
    const debugMenu = dolphin_js_1.Gui.MainWindow.getMenuBar().addMenu('Debug');
    const toggleImGuiAction = debugMenu.addAction('ImGui');
    toggleImGuiAction.checkable = true;
    toggleImGuiAction.setToggledCallback((c) => {
        worker_threads_1.default.parentPort?.postMessage({ msg: 'toggleImGuiVisibility', data: c });
    });
    toggleImGuiAction.checked = false;
    toggleImGuiAction.setShortcut('Ctrl+I');
    worker_threads_1.default.parentPort?.on('message', value => {
        if (value.msg == 'notifyHideImGui')
            toggleImGuiAction.checked = false;
    });
    dolphin_js_1.Gui.MainWindow.startGame(startInfo.gameFilePath);
}
const helpMenu = dolphin_js_1.Gui.MainWindow.findMenu('Help');
if (helpMenu) {
    const aboutAction = helpMenu.addAction('About ModLoader64');
    aboutAction.setTriggeredCallback(() => {
        dolphin_js_1.Gui.Q.CommonDialogs.about(dolphin_js_1.Gui.MainWindow.asWidget(), 'About ModLoader64', 'ModLoader64 is a network capable mod loading system for Nintendo 64 and GameCube games.<br/>' +
            'Its main purpose is creating online multiplayer mods for various games like Ocarina of Time.<br/>' +
            '<a href="https://modloader64.com/">Website</a> <a href="https://discord.gg/nHb4fXX">Discord</a>');
    });
}
worker_threads_1.default.parentPort?.postMessage({ msg: 'hostReady' });
//# sourceMappingURL=DolphinHostThread.js.map