"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DolphinMemory = void 0;
const dolphin_js_1 = require("dolphin-js");
class DolphinMemory {
    rdramRead8(addr) {
        return dolphin_js_1.Memory.readU8(addr);
    }
    rdramWrite8(addr, value) {
        dolphin_js_1.Memory.writeU8(addr, value);
    }
    rdramRead16(addr) {
        return dolphin_js_1.Memory.readU16BE(addr);
    }
    rdramWrite16(addr, value) {
        dolphin_js_1.Memory.writeU16BE(addr, value);
    }
    rdramWrite32(addr, value) {
        dolphin_js_1.Memory.writeU32BE(addr, value);
    }
    rdramRead32(addr) {
        return dolphin_js_1.Memory.readU32BE(addr);
    }
    rdramReadBuffer(addr, size) {
        return Buffer.from(dolphin_js_1.Memory.readBufferU8(addr, size));
    }
    rdramWriteBuffer(addr, buf) {
        dolphin_js_1.Memory.writeBufferU8(addr, buf);
    }
    dereferencePointer(addr) {
        return 0;
    }
    rdramReadS8(addr) {
        return dolphin_js_1.Memory.readS8(addr);
    }
    rdramReadS16(addr) {
        return dolphin_js_1.Memory.readS16BE(addr);
    }
    rdramReadS32(addr) {
        return dolphin_js_1.Memory.readS32BE(addr);
    }
    rdramReadBitsBuffer(addr, bytes) {
        return Buffer.from(dolphin_js_1.Memory.readBitsBufferU8(addr, bytes));
    }
    rdramReadBits8(addr) {
        //return Buffer.from(bitwise.byte.read(this.rdramRead8(addr) as any));
        return Buffer.from(dolphin_js_1.Memory.readBitsU8(addr));
    }
    rdramReadBit8(addr, bitoffset) {
        return dolphin_js_1.Memory.readBitU8(addr, bitoffset);
    }
    rdramWriteBitsBuffer(addr, buf) {
        dolphin_js_1.Memory.writeBitsBufferU8(addr, buf);
    }
    // @Sylvain - writeBitsU8 appears to be broken.
    rdramWriteBits8(addr, buf) {
        /*         let arr = [...buf];
                let b = bitwise.byte.write(arr as any);
                this.rdramWrite8(addr, b); */
        dolphin_js_1.Memory.writeBitsU8(addr, buf);
    }
    rdramWriteBit8(addr, bitoffset, bit) {
        dolphin_js_1.Memory.writeBitU8(addr, bitoffset, bit);
    }
    rdramReadPtr8(addr, offset) {
        return dolphin_js_1.Memory.readPtrU8(addr, offset);
    }
    rdramWritePtr8(addr, offset, value) {
        dolphin_js_1.Memory.writePtrU8(addr, offset, value);
    }
    rdramReadPtr16(addr, offset) {
        return dolphin_js_1.Memory.readPtrU16BE(addr, offset);
    }
    rdramWritePtr16(addr, offset, value) {
        dolphin_js_1.Memory.writePtrU16BE(addr, offset, value);
    }
    rdramWritePtr32(addr, offset, value) {
        dolphin_js_1.Memory.writePtrU32BE(addr, offset, value);
    }
    rdramReadPtr32(addr, offset) {
        return dolphin_js_1.Memory.readPtrU32BE(addr, offset);
    }
    rdramReadPtrBuffer(addr, offset, size) {
        return Buffer.from(dolphin_js_1.Memory.readPtrBufferU8(addr, offset, size));
    }
    rdramWritePtrBuffer(addr, offset, buf) {
        dolphin_js_1.Memory.writePtrBufferU8(addr, offset, buf);
    }
    rdramReadPtrS8(addr, offset) {
        return dolphin_js_1.Memory.readPtrS8(addr, offset);
    }
    rdramReadPtrS16(addr, offset) {
        return dolphin_js_1.Memory.readPtrS16BE(addr, offset);
    }
    rdramReadPtrS32(addr, offset) {
        return dolphin_js_1.Memory.readPtrS32BE(addr, offset);
    }
    rdramReadPtrBitsBuffer(addr, offset, bytes) {
        return Buffer.from(dolphin_js_1.Memory.readPtrBitsBufferU8(addr, offset, bytes));
    }
    rdramReadPtrBits8(addr, offset) {
        return Buffer.from(dolphin_js_1.Memory.readPtrBitsU8(addr, offset));
    }
    rdramReadPtrBit8(addr, offset, bitoffset) {
        return dolphin_js_1.Memory.readPtrBitU8(addr, offset, bitoffset);
    }
    rdramWritePtrBitsBuffer(addr, offset, buf) {
        dolphin_js_1.Memory.writePtrBitsBufferU8(addr, offset, buf);
    }
    rdramWritePtrBits8(addr, offset, buf) {
        dolphin_js_1.Memory.writePtrBitsU8(addr, offset, buf);
    }
    rdramWritePtrBit8(addr, offset, bitoffset, bit) {
        dolphin_js_1.Memory.writePtrBitU8(addr, offset, bitoffset, bit);
    }
    rdramReadF32(addr) {
        return dolphin_js_1.Memory.readF32BE(addr);
    }
    rdramReadPtrF32(addr, offset) {
        return dolphin_js_1.Memory.readPtrF32BE(addr, offset);
    }
    rdramWriteF32(addr, value) {
        dolphin_js_1.Memory.writeF32BE(addr, value);
    }
    rdramWritePtrF32(addr, offset, value) {
        dolphin_js_1.Memory.writePtrF32BE(addr, offset, value);
    }
    bitCount8(value) {
        return 0;
    }
    bitCount16(value) {
        return 0;
    }
    bitCount32(value) {
        return 0;
    }
    bitCountBuffer(buf, off, len) {
        return 0;
    }
    getRdramBuffer() {
        return new ArrayBuffer(0);
    }
    rdramRead64(addr) {
        return 0;
    }
    rdramReadS64(addr) {
        return 0;
    }
    rdramReadF64(addr) {
        return dolphin_js_1.Memory.readF64BE(addr);
    }
    rdramWrite64(addr, val) {
        dolphin_js_1.Memory.writeU64BE(addr, val);
    }
    rdramWriteF64(addr, val) {
        dolphin_js_1.Memory.writeF64BE(addr, val);
    }
    rdramReadBigInt64(addr) {
        throw new Error('Method not implemented.');
    }
    rdramReadBigIntS64(addr) {
        throw new Error('Method not implemented.');
    }
    rdramWriteBigInt64(addr, val) {
        throw new Error('Method not implemented.');
    }
    u8 = [];
    u16 = [];
    u32 = [];
    u64 = [];
    s8 = [];
    s16 = [];
    s32 = [];
    s64 = [];
    f32 = [];
    f64 = [];
    invalidateCachedCode(address, size, forced) {
        dolphin_js_1.JitInterface.clearCache();
    }
}
exports.DolphinMemory = DolphinMemory;
//# sourceMappingURL=DolphinMemory.js.map