"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImGuiApp = void 0;
const ml64tk_1 = require("ml64tk");
class ImGuiApp {
    appWindow;
    darkMode = false;
    constructor(title, viewports) {
        this.appWindow = new ml64tk_1.AppWindow(false, viewports ? viewports : false);
        this.appWindow.on('init', this.init.bind(this));
        this.appWindow.on('new-frame', this.newFrame.bind(this));
        this.appWindow.on('close', this.onClose.bind(this));
        this.appWindow.title = title;
        this.appWindow.clearColor = ml64tk_1.$.rgbaf(0, 0, 0, 1);
    }
    run() {
        const process = setInterval((() => {
            if (this.appWindow.doIteration()) {
                clearInterval(process);
                this.onClosed();
            }
        }).bind(this));
    }
    init() {
        this.onInit();
    }
    newFrame() {
        if (!this.darkMode) {
            this.darkMode = true;
            if (process.platform == 'win32') {
                // @ts-ignore
                ml64tk_1.Gui.useImmersiveDarkMode(ml64tk_1.ImGui.getWindowViewport(), true);
            }
        }
        this.createMainDockSpace();
        this.onNewFrame();
    }
    createMainDockSpace() {
        const mainViewport = ml64tk_1.ImGui.getMainViewport();
        ml64tk_1.ImGui.setNextWindowViewport(mainViewport.id);
        ml64tk_1.ImGui.setNextWindowPos(mainViewport.workPos, 1 /* ImGui.Cond.Always */);
        ml64tk_1.ImGui.setNextWindowSize(mainViewport.workSize, 1 /* ImGui.Cond.Always */);
        ml64tk_1.ImGui.pushStyleVar(2 /* ImGui.StyleVar.WindowPadding */, ml64tk_1.$.xy(0, 0));
        ml64tk_1.ImGui.pushStyleColor(39 /* ImGui.Col.DockingEmptyBg */, ml64tk_1.$.rgbaf(0, 0, 0, 0));
        ml64tk_1.ImGui.begin("##MainWindowDockSpace", undefined, 786432 /* ImGui.WindowFlags.NoNav */ | 43 /* ImGui.WindowFlags.NoDecoration */
            | 256 /* ImGui.WindowFlags.NoSavedSettings */ | 128 /* ImGui.WindowFlags.NoBackground */
            | 8192 /* ImGui.WindowFlags.NoBringToFrontOnFocus */ | 4096 /* ImGui.WindowFlags.NoFocusOnAppearing */);
        ml64tk_1.ImGui.dockSpace(ml64tk_1.ImGui.getId("MainDockSpace"));
        ml64tk_1.ImGui.end();
        ml64tk_1.ImGui.popStyleColor();
        ml64tk_1.ImGui.popStyleVar();
    }
    onInit() {
    }
    onNewFrame() {
    }
    onClose() {
        return true;
    }
    onClosed() {
    }
}
exports.ImGuiApp = ImGuiApp;
//# sourceMappingURL=ImGuiApp.js.map